<?php
namespace Flat101\LimitingRegionsCheckout\Model\Config\Region;

use Magento\Directory\Api\CountryInformationAcquirerInterface;
use Magento\Directory\Model\Country;
use Magento\Directory\Model\ResourceModel\Country\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

class RegionInformationProvider
{
    /** @var CountryInformationAcquirerInterface */
    protected $_countryCollectionFactory;

    /** @var RequestInterface */
    protected $request;

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * RegionInformationProvider constructor.
     * @param CollectionFactory $_countryCollectionFactory
     * @param Country $country
     * @param ScopeConfigInterface $scopeConfig
     * @param RequestInterface $request
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        CollectionFactory $_countryCollectionFactory,
        Country $country,
        ScopeConfigInterface $scopeConfig,
        RequestInterface $request,
        StoreManagerInterface $storeManager
    ) {
        $this->_countryCollectionFactory = $_countryCollectionFactory;
        $this->country = $country;
        $this->scopeConfig = $scopeConfig;
        $this->request = $request;
        $this->_storeManager = $storeManager;
    }

    /**
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function toOptionArray()
    {
        $websiteId = (int) $this->request->getParam('website');

        if ($websiteId === 0) {
            // If we are in the default website we look up the one we need using the store id
            $storeId = (int) $this->request->getParam('store');
            $websiteId = $this->_storeManager->getStore($storeId)->getWebsiteId();
        }

        // Get regions for this country
        $allowedCountries = $this->scopeConfig->getValue('general/country/allow', ScopeInterface::SCOPE_WEBSITES, $websiteId);

        $allowedCountries = explode(",", $allowedCountries);
        $regions = [];
        foreach ($allowedCountries as $country) {
            $regionCollection = $this->country->loadByCode($country)->getRegions();
            $availableRegions = $regionCollection->loadData()->toArray();

            foreach ($availableRegions['items'] as $region) {
                $regions[] = [
                    'value' => $region['region_id'],
                    'label' => $region['name']
                ];
            }
        }

        return $regions;
    }
}
