<?php

namespace Flat101\LimitingRegionsCheckout\Plugin;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;

class AllowedRegionsCheckout
{
    /** @var ScopeConfigInterface */
    protected $scopeConfig;
    protected $allowedRegions;

    /**
     * StateFilter constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    public function afterGetRegionData($subject, $regions)
    {
        $allowedRegions = $this->scopeConfig->getValue('general/country/allowed_regions', ScopeInterface::SCOPE_STORE);
        $this->allowedRegions = explode(",", $allowedRegions);
        foreach ($regions['config']['regions_required'] as $country) {
            if (isset($regions[$country])) {
                $regions[$country] = array_filter($regions[$country], function ($value, $key) {
                    return in_array($key, $this->allowedRegions);
                }, ARRAY_FILTER_USE_BOTH);
            }
        }

        return $regions;
    }

    public function afterToOptionArray($subject, $options)
    {
        $allowedRegions = $this->scopeConfig->getValue('general/country/allowed_regions', ScopeInterface::SCOPE_STORE);
        $this->allowedRegions = explode(",", $allowedRegions);
        $result = array_filter($options, function ($option) {
            if (isset($option['value'])) {
                return in_array($option['value'], $this->allowedRegions);
            }
            return true;
        });
        if (count($result) > 0) {
            array_unshift(
                $result,
                ['title' => '', 'value' => '', 'label' => __('Please select a region, state or province.')]
            );
        }
        return $result;
    }
}
